enum err_type {WRONG_EOF,WRONG_SYNTAX,CONSTANT_GATE};
#define ERROR 1
#define MAX_NAME 100
#define p(){printf("%s, line %d\n",__FILE__,__LINE__);}

typedef Minisat::vec<Minisat::Lit> TrivMclause;
typedef Minisat::Lit Mlit;
typedef TrivMclause MvecLits;
typedef Minisat::Clause ComplMclause;
typedef Minisat::lbool Mbool;
typedef Minisat::vec<Mbool> MboolVec;
typedef Minisat::Solver Msolver;
const Mbool Mtrue = Minisat::l_True;
const Mbool Mfalse = Minisat::l_False;
const Mbool Mundef = Minisat::l_Undef;

typedef vector<int> CUBE;
typedef vector<CUBE> DNF;
typedef vector<char> CCUBE;
typedef set <int> SCUBE;
typedef CUBE CLAUSE;
typedef DNF CNF;
typedef unsigned int uint;

const int BUF_SIZE = 10000;

/*=================

    VerPqe

 ===============*/
class VerPqe {
public:

  CNF F; // initial formula
  CUBE Yvars; // free i.e. unquantified variables
  CUBE Trgs; // target clauses of F
  CNF Sol; // solution of the pqe problem specified by F,Yvars and Trgs
  
  CNF F1; // the subset of F consisting of target cubes
  CNF F2; // the set F \ F1
  
 
  CCUBE Xvar; // Xvar[i] = 0 (1) if variable i+1 is in Yvars
             // (respect. not in Yvars)
 
  int num_vars; // number of variables in N (and F)
  
  Msolver *Slvr; // operates on formula F to generate a blocking clause excluding
                 // an assignment satisfying F
  DNF Flits0; // Flits0[i] specifies the cubes of F with the negative literal of var(i)
  DNF Flits1; // Flits1[i] specifies the cubes of F with the positive literal of var(i)
  

  //
  //  Methods
  //
  void print_err_header(FILE *fp,char *root);
  int check_sol();
  void print_err_report1(char *root,int ind);
  void print_symb_cube(FILE *fp,CUBE &A,bool cr = true);
  void print_err_report2(char *root,CUBE &Rbp);
  void drop_yvars(CNF &A,CNF &B);
  bool mandatory(CUBE &A,int assgn,SCUBE &Ya,int cube_ind);
  void form_blocking_clause(CUBE &B,SCUBE &Ya);
  void reduce_y_assgns(SCUBE &Ya,CUBE &A);
  void extr_all_assgns(CUBE &A,Msolver *Slvr);
  bool removable_bnd_pnt(Msolver *Aux_slvr);
  void extr_y_assgns(CUBE &Ya,Msolver *Aux_slvr);
  void blocking_clause(CLAUSE &B);
  bool prove_red(CUBE &Rbp,CUBE &C);
  bool verify(char *fname);
  void read_input(char *root);
  void init();
};



//
//   Function propotypes
//

void get_runtime (double &usrtime, double &systime);
void initialize_cube_vects(DNF &A,DNF & Lits0,DNF &Lits1);
int find_max_var(DNF &D);
int read_dimacs1(FILE *fp,DNF &D,int &num_vars);
void read_numbers(const char *fname,CUBE &Sol);
Msolver *init_sat_solver(int nvars);
void conv_to_mclause(TrivMclause &A, CLAUSE &C);
void accept_new_clause(Msolver *Slv,CLAUSE &C);
void add_assumps(MvecLits &Assmps,CUBE &A);
void add_negated_assumps(MvecLits &Assmps,CLAUSE &C);
void accept_new_clauses(Msolver *Slvr,CNF &H);
