#include <iostream>
#include <list>
#include <vector>
#include <set>
#include <map>
#include <string>
#include <queue>
#include <algorithm>
//
#include <stdio.h>
#include <assert.h>
#include <string.h>

#include "Solver.h"
#include "SimpSolver.h"
using namespace std;

#include "d0efs.hh"

//
//  PRINT_ERR_HEADER
//
void VerPqe::print_err_header(FILE *fp,char *root)
{

  fprintf(fp,"PQE: Let G(X,Y) be a subset of F(X,Y). Find G*(Y) s.t.\n");
  fprintf(fp,"     Exists(X)[F] = G* & Exists(X)[F \\ G]\n");
  fprintf(fp," ------------\n");
  fprintf(fp,"  F is specified by %s.cnf\n",root);
  fprintf(fp,"  G is specified by %s.clauses\n",root);
  fprintf(fp,"  Y  is specified by %s.txt\n",root);
  fprintf(fp," ------------\n");
  fprintf(fp,"  G* is specified by %s.res.cnf\n",root);
  

} /* print_err_header */



//
// PRINT_SYMB_CUBE
//
void VerPqe::print_symb_cube(FILE *fp,CUBE &A,bool cr)
{
  SCUBE Vars,Lits;

  
  for (uint i=0; i < A.size(); i++) {
    Vars.insert(abs(A[i]));
    Lits.insert(A[i]);
  }

  int count = 0;
  for (int i=0; i < num_vars; i++) {
    if (Vars.find(i+1) == Vars.end()) continue;
    if (count > 0) fprintf(fp," ");
    count++;
    int lit = i+1;
    if (Lits.find(-lit) != Lits.end())
      lit = -lit;
    if (lit < 0) fprintf(fp,"~");
    if (Xvar[i]) fprintf(fp,"x");
    else fprintf(fp,"y");
    fprintf(fp,"%d",i+1);
  }

  if (cr) fprintf(fp,"\n");
} /* print_symb_cube */

//
//   PRINT_ERR_REPORT2
//
void VerPqe::print_err_report2(char *root,CUBE &Rbp)
{


  CUBE X_rbp,Y_rbp;
  for (uint i=0; i < Rbp.size(); i++) {
    int var_ind = abs(Rbp[i])-1;
    if (Xvar[var_ind]) X_rbp.push_back(-Rbp[i]);
    else Y_rbp.push_back(-Rbp[i]);
  }
   
  char fname[MAX_NAME];
  strcpy(fname,root);

  strcat(fname,".report");
  
  FILE *fp = fopen(fname,"w");
  assert(fp != NULL);

  print_err_header(fp,root);

  fprintf(fp,"\n ====================\n");
  fprintf(fp,"   error report\n");
  fprintf(fp," ====================\n");
  fprintf(fp,"Exists(X)[F] != G* & Exists(X)[F \\ G]\n");
  fprintf(fp,"LHS = 0 and RHS = 1\n");
  if (Y_rbp.size() > 0) {
    fprintf(fp, "in subspace ");
    print_symb_cube(fp,Y_rbp);
    fprintf(fp,"\n");
  }
  fprintf(fp,"An assignment satisfying RHS");
  if (Y_rbp.size() > 0) fprintf(fp," in this subspace :\n");
  else fprintf(fp,"\n");
  fprintf(fp,"  ");
  print_symb_cube(fp,Y_rbp,false);
  fprintf(fp," ");
  print_symb_cube(fp,X_rbp);
  fclose(fp);
  

} /* print_err_report2 */


//
//  DROP_YVARS
//
void VerPqe::drop_yvars(CNF &A,CNF &B)
{

  for (uint i=0; i < B.size(); i++) {
    CLAUSE &C1 = B[i];
    CLAUSE C2;
    for (uint j=0; j < C1.size(); j++) {
      int var_ind = abs(C1[j])-1;
      if (Xvar[var_ind] == 0) continue;
      C2.push_back(C1[j]);
    }
    A.push_back(C2);
  }

} /* drop_yvars */

