#include <iostream>
#include <list>
#include <vector>
#include <set>
#include <map>
#include <string>
#include <queue>
#include <algorithm>
//
#include <stdio.h>
#include <assert.h>
#include <string.h>

#include "Solver.h"
#include "SimpSolver.h"
using namespace std;

#include "d0efs.hh"


/*===================

  M A I N

  ==================*/
int main(int argc,char *argv[])
{

  VerPqe  Vp;

  if (argc == 1)  {
    printf("ver_pqe root\n");
    printf("   the PQE problem is specified by root.cnf, root.txt, root.clauses\n");
    printf("   a solution of this problem is specified by root.res.cnf\n");
    exit(0);
  }

  if (argc != 2)   {
    printf("wrong number of arguments: %d\n",argc);
    exit(ERROR); }

  Vp.read_input(argv[1]);
  Vp.init();

  double usrtime0=0.,systime0=0.;
  double usrtime=0.,systime=0.;
  get_runtime (usrtime0, systime0);
   
  bool ok = Vp.verify(argv[1]);

  get_runtime (usrtime, systime);
  printf("total time is %.2f sec.\n",usrtime-usrtime0);
  
  if (ok) {
    printf("verification is ok\n");
    exit(0);
  }
  else {
    printf("verification failed\n");
    printf("  see the details in %s.report\n",argv[1]);
    exit(1);
  }
  
}/* end of function main */


//
//  VERIFY
//
bool VerPqe::verify(char *fname)
{

  int err_cls_ind = check_sol();
  if (err_cls_ind >= 0) {
    print_err_report1(fname,err_cls_ind);
    return(false);
  }
  for (uint i=0; i < Sol.size(); i++)
    if (Sol[i].size() == 0) return(true);

  while (F1.size() > 0) {
    CLAUSE C = F1.back();
    F1.pop_back();
    CUBE Rbp;
    bool ok = prove_red(Rbp,C);
    if (!ok) {
      print_err_report2(fname,Rbp);
      return(false);
    }
  }

  return(true);

} /* verify */


//
//  READ_INPUT
//
void VerPqe::read_input(char *root)
{  
  char fname[MAX_NAME];
  strcpy(fname,root);

  // read CNF formula
  strcat(fname,".cnf");
  FILE *fp1 = fopen(fname,"r");
  if (fp1 == NULL) {
    p();
    exit(ERROR);
  }
  
  read_dimacs1(fp1,F,num_vars);
  
  // read list of free (i.e. unquantified) variables
  fname[0] = 0;
  strcpy(fname,root);
  strcat(fname,".txt");
  read_numbers(fname,Yvars);       
 
  
  // read list of cubes to be made redundant
  fname[0] = 0;
  strcpy(fname,root);
  strcat(fname,".clauses");
  read_numbers(fname,Trgs);

  // read solution
  fname[0] = 0;
  strcpy(fname,root);
  strcat(fname,".res.cnf");
  FILE *fp2 = fopen(fname,"r");
  if (fp2 == NULL) {
    p();
    exit(ERROR);
  }
  int tmp;
  read_dimacs1(fp2,Sol,tmp);
  int max_var = find_max_var(Sol);
  if (max_var > num_vars) {
    p();
    printf("max_var = %d, num_vars = %d\n",max_var,num_vars);
    exit(ERROR);
  }
  
}  /* read_input */


//
//   INIT
//
void VerPqe::init()
{
  
  Xvar.assign(num_vars,1);
  for (uint i=0; i < Yvars.size(); i++)
    Xvar[Yvars[i]-1] = 0;

  SCUBE S;
  for (uint i=0; i < Trgs.size(); i++)
    S.insert(Trgs[i]);

  for (uint i=0; i < F.size(); i++)
    if (S.find(i) != S.end()) F1.push_back(F[i]);
    else F2.push_back(F[i]);


  for (int i=0; i < num_vars; i++)    {
    CUBE  Dummy;
    Flits0.push_back(Dummy);
    Flits1.push_back(Dummy);
  }

  // initialize literal counts  
  initialize_cube_vects(F,Flits0,Flits1);

  Slvr = init_sat_solver(num_vars);
  accept_new_clauses(Slvr,F);
  
} /* init */


