#include <iostream>
#include <list>
#include <vector>
#include <set>
#include <map>
#include <string>
#include <queue>
#include <algorithm>
//
#include <stdio.h>
#include <assert.h>
#include <string.h>

#include "Solver.h"
#include "SimpSolver.h"
using namespace std;

#include "d0efs.hh"

//
//  PROVE_RED
//
bool VerPqe::prove_red(CUBE &Rbp,CLAUSE &C)
{

  Msolver *Aux_slvr = init_sat_solver(num_vars);

  accept_new_clauses(Aux_slvr,F1);
  accept_new_clauses(Aux_slvr,F2);
  accept_new_clauses(Aux_slvr,Sol);

  while (true) {
    MvecLits Assmps;
    add_negated_assumps(Assmps,C);
    bool sat = Aux_slvr->solve(Assmps);
 
    if (!sat) {
      delete Aux_slvr;
      return(true);
    }
  
    bool yes = removable_bnd_pnt(Aux_slvr);
    if (yes) {
      extr_all_assgns(Rbp,Aux_slvr);
      for (uint i=0; i < Rbp.size(); i++)
        Rbp[i] = -Rbp[i];
      delete Aux_slvr;
      return(false);
    }
    CLAUSE B;
    blocking_clause(B);
    if (B.size() == 0) {
      delete Aux_slvr;
      return(true);
    }
   
    accept_new_clause(Aux_slvr,B); 
  }

} /* prove_red */


//
//  FORM_BLOCKING_CLAUSE
//
void VerPqe::form_blocking_clause(CUBE &B,SCUBE &Ya)
{

  SCUBE::iterator pnt;

  for (pnt = Ya.begin(); pnt != Ya.end(); pnt++)
    B.push_back(-(*pnt));

} /* form_blocking_clause */

//
// MANDATORY
//
bool VerPqe::mandatory(CUBE &A,int assgn,SCUBE &Ya,int cls_ind)
{

  CLAUSE &C = F[cls_ind];

  
  for (uint i=0; i < C.size(); i++) {
    if (C[i] == assgn) continue;
    int var_ind = abs(C[i])-1;
    if (A[var_ind] == -C[i]) continue; // skip falsif. assgn
    if (Xvar[var_ind]) return(false); // sat by an x-assgn
    if (Ya.find(A[var_ind]) != Ya.end())
      return(false); // sat by an y-assgn of Ya
  }

  return(true);
  
} /* mandatory */

//
//  REDUCE_Y_ASSGNS
//
void VerPqe::reduce_y_assgns(SCUBE &Ya,CUBE &A)
{

  for (uint i=0; i < A.size(); i++) {
    if (Xvar[i]) continue;
    CUBE *pClauses;
    if (A[i] < 0) pClauses = &Flits0[i];
    else pClauses = &Flits1[i];

    bool crucial = false;
    for (uint j=0; j < pClauses->size(); j++) {
      int cls_ind = (*pClauses)[j];
      if (mandatory(A,A[i],Ya,cls_ind)) {
        crucial = true;
        break;
      }
    }

    if (crucial) Ya.insert(A[i]);
  }

} /* reduce_y_assgns */

//
// BLOCKING_CLAUSE
//
void VerPqe::blocking_clause(CLAUSE &B)
{

 CUBE A;

 extr_all_assgns(A,Slvr);
 assert((int) A.size() == num_vars);
 SCUBE Ya;

 reduce_y_assgns(Ya,A);
 form_blocking_clause(B,Ya);

} /* blocking_clause */
//
//  REMOVABLE_BND_PNT
//
bool VerPqe::removable_bnd_pnt(Msolver *Aux_slvr)
{

  CUBE Ya;

  extr_y_assgns(Ya,Aux_slvr);
  MvecLits Assmps;
  add_assumps(Assmps,Ya);
  bool sat = Slvr->solve(Assmps);
  return(!sat); // if (!sat) the boundary point is removable

} /* removable_bnd_pnt */




//
//  EXTR_Y_ASSGNS
//
void VerPqe::extr_y_assgns(CUBE &Ya,Msolver *Aux_slvr)
{
  MboolVec &S = Aux_slvr->model;
  for (int i=0; i < num_vars; i++) {
    if (Xvar[i]) continue;
    if (S[i] == Mtrue) Ya.push_back(i+1);
    else Ya.push_back(-(i+1));
  }
} /* extr_y_assgns */


//
//  EXTR_ALL_ASSGNS
//
void VerPqe::extr_all_assgns(CUBE &A,Msolver *Slvr)
{
  MboolVec &S = Slvr->model;
  for (int i=0; i < num_vars; i++) 
    if (S[i] == Mtrue) A.push_back(i+1);
    else A.push_back(-(i+1));
} /* extr_all_assgns */

