#include <iostream>
#include <list>
#include <vector>
#include <set>
#include <map>
#include <string>
#include <queue>
#include <algorithm>
//
#include <stdio.h>
#include <assert.h>
#include <string.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <unistd.h>

#include "Solver.h"
#include "SimpSolver.h"
using namespace std;

#include "d0efs.hh"

/*==============================================

  I N I T I A L I Z E _ C U B E _ V E C T S

  ===============================================*/
void initialize_cube_vects(DNF &A,DNF & Lits0,DNF &Lits1)

{unsigned int i,j;
  int lit;


  for (i=0; i < A.size();i++) {
    CUBE &cube = A[i];
    for (j = 0; j < cube.size();j++) {
      lit = cube[j];
      if (lit < 0)
        Lits0[(-1*lit)-1].push_back(i); // add cube index
      else
        Lits1[lit-1].push_back(i); // add cube index
    }
  }
} /* end of function initialize_cube_vects */
/*=======================================

      I N I T _ S A T _ S O L V E R

  ======================================*/
Msolver *init_sat_solver(int nvars)
{
 Msolver *S = new Minisat::Solver();

  for (int i = 0; i < nvars; ++i) 
    S->newVar();

  return(S);

} /* end of function init_sat_solver */

/*=================================

  C O N V _ T O _ M C L A U S E

  ===============================*/
void conv_to_mclause(TrivMclause &A, CLAUSE &C) {

  for (uint i=0; i < C.size(); i++) {
    if (C[i] > 0) A.push(Minisat::mkLit(C[i]-1,false));
    else A.push(Minisat::mkLit(-C[i]-1,true));
  }

} /* end of function conv_to_mclause */

/*=======================================

  A C C E P T _ N E W _ C L A U S E

  ========================================*/
void accept_new_clause(Msolver *Slvr,CLAUSE &C)
{
 
  TrivMclause A;
  conv_to_mclause(A,C);
  Slvr->addClause(A);


}  /* end of function accept_new_clause */

/*=============================

     A D D _ A S S U M P S 

  ============================*/
void add_assumps(MvecLits &Assmps,CUBE &A)
{

  for (uint i=0; i < A.size(); i++) {
    int lit = A[i];
    if (lit < 0) Assmps.push(Minisat::mkLit(-lit-1,true));
    else Assmps.push(Minisat::mkLit(lit-1,false));
  }
    
} /* end of function add_assumps */


/*===========================================

     A D D _ N E G A T E D _ A S S U M P S

  ==========================================*/
void add_negated_assumps(MvecLits &Assmps,CLAUSE &C)
{

  for (uint i=0; i < C.size(); i++) {
    int lit = C[i];
    if (lit < 0) Assmps.push(Minisat::mkLit(-lit-1,false));
    else Assmps.push(Minisat::mkLit(lit-1,true));
  }
    
} /* end of function add_negated_assumps*/

/*==========================================

  A C C E P T _ N E W _ C L A U S E S

  =========================================*/
void accept_new_clauses(Msolver *Slvr,CNF &H)
{
  for (uint i=0; i < H.size(); i++)
    accept_new_clause(Slvr,H[i]);

} /* end of function accept_new_clauses */


/*==========================================================

   G E T _ R U N T I M E

  the function compute the sytem and user time spent by the 
  process
  
============================================================*/
 void get_runtime (double &usrtime, double &systime)
{
  struct rusage timeusage;

  getrusage(RUSAGE_SELF, &timeusage);

  usrtime = (double)timeusage.ru_utime.tv_sec  + (double)timeusage.ru_utime.tv_usec / 1000000;
  systime = (double)timeusage.ru_stime.tv_sec  + (double)timeusage.ru_stime.tv_usec / 1000000;
} /* end of function get_runtime */
